{{
 Demo of i2cDriver.spin
 Erlend Fj 2015
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------
 Executes I2C bus commands such as Initiate, WhoOnBus, Read, and Write - provided through the PST serial terminal
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------
}}
{
 Acknowledgements:  zz

==================================================================
 Hardware & wiring:


 Data sheet: <link>

=======================================================================================================================================================================
}

CON

          _clkmode = xtal1 + pll16x
          _xinfreq = 5_000_000                                          ' use 5MHz crystal
        
          clk_freq = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
          mSec     = clk_freq / 1_000                                   ' ticks in 1ms
          uSec     = clk_freq / 1_000_000                               ' ticks in 1us
        
                
VAR
           BYTE  BusInitilized                                            'Flag to store if init is done
           LONG  OnBus[120] '                                              'Hold list of slaves detected, and tot number in [0]
        

OBJ
           bus   : "i2cDriver"
           pst   : "Parallax Serial Terminal"
          

PUB Main | value                                  

  pst.Start(115200)
  WAITCNT((2*(clkfreq/1000)) + cnt)
  value := pst.DecIn                                                      ' Start the Parallax Serial Terminal cog
  
  BusInitilized:= FALSE  
  
  pst.Str(String("Testing of I2C driver and bus."))                             ' Heading
  repeat                                                                        ' Main loop
    pst.Chars(pst#NL, 2)                                                        ' Carriage returns
    pst.Str(String("Enter 0-Initiate, 1-WhoOnBus, 2-Read, 3-Write, -1-Quit: ")) ' Prompt user to enter value
    value := pst.DecIn                                                          ' Get value

    CASE value
        0  : Initiate
        1  : ScanBus
        2  : Read
        3  : Write
        -1 : QUIT
      OTHER: pst.Str(String("Sorry - invalid input."))
    
      
  pst.Str(String(pst#NL,"Bye."))                  
                                                           ' Display hexadecimal value


PUB Initiate | PINscl, PINsda

    REPEAT
      pst.Str(String("Enter SCL pin number:"))
      PINscl:= || pst.DecIn
      pst.Chars(pst#NL, 2)
      pst.Str(String("Enter SDA pin number:"))
      PINsda:= || pst.DecIn
      pst.Chars(pst#NL, 2)
      IF PINscl>31 OR PINsda>31
        pst.Str(String("Pin number maximum 31!"))
        pst.Chars(pst#NL, 2)
      ELSE
        QUIT
        
    pst.Str(String("Now initialising bus..."))    
    bus.Init(PINscl, PINsda)
    pst.Str(String("Status of SDA: "))
    pst.Bin(INA[PINsda], 1)
    pst.Chars(pst#NL, 2)
    pst.Str(String("Completed."))
    BusInitilized:= TRUE

    
PUB ScanBus | i
 
   IF NOT BusInitilized
     pst.Chars(pst#NL, 2)
     pst.Str(String("Bus not initialized, will do that first..."))
     pst.Chars(pst#NL, 2)     
     Initiate
       
   pst.Chars(pst#NL, 2)
   pst.Str(String("Now scanning bus..."))
   pst.Chars(pst#NL, 2)    
   bus.WhoOnBus(@OnBus)
   
   IF OnBus[0] > 0
     REPEAT i FROM 1 TO OnBus[0]
       pst.Str(String("Bus slave detected at address: $"))
       pst.Hex(OnBus[i], 4)
       pst.Chars(pst#NL, 2)
   pst.Dec(OnBus[0])
   pst.Str(String("  slaves found.  "))
   pst.Str(String("Scan completed."))   
   pst.Chars(pst#NL, 2)
   

PUB Read | ChipAddr, RegAddr, AddrForm, value 

    REPEAT
      pst.Str(String("Enter Chip address (HEX):"))
      ChipAddr:= pst.HexIn
      pst.Chars(pst#NL, 2)
      pst.Str(String("Enter Register address (HEX):"))
      RegAddr:= pst.HexIn
      pst.Str(String("-1-Quit, 1-8bit format, 2-16bit format, 3-32bit format:"))
      AddrForm:= pst.DecIn      
      pst.Chars(pst#NL, 2)
   
      CASE AddrForm
          1  : value:= bus.ReadChipA8(ChipAddr, RegAddr, bus#NAK)
          2  : value:= bus.ReadChipA16(ChipAddr, RegAddr, bus#NAK)
          3  : value:= bus.ReadChipA32(ChipAddr, RegAddr, bus#NAK)
          -1 : RETURN
        OTHER: pst.Str(String("Chose -1, 1, 2, or 3!"))
               pst.Chars(pst#NL, 2)
               
   pst.Str(String("Reply from chip: $"))    
   pst.Hex(value, 4)
   pst.Str(String("   %"))
   pst.Bin(value, 32)
   pst.Str(String("   Dec: "))
   pst.Dec(value)
   pst.Chars(pst#NL, 2)
   pst.Str(String("Completed."))   
   pst.Chars(pst#NL, 2)      

      
PUB Write | ChipAddr, RegAddr, AddrForm, Value 

    REPEAT
      pst.Str(String("Enter Chip address (HEX):"))
      ChipAddr:= pst.HexIn
      pst.Chars(pst#NL, 2)
      pst.Str(String("Enter Register address (HEX):"))
      RegAddr:= pst.HexIn
      pst.Str(String("Enter value to be written (HEX):"))
      RegAddr:= pst.HexIn      
      pst.Str(String("-1-Quit, 1-8bit format, 2-16bit format, 3-32bit format:"))
      AddrForm:= pst.DecIn      
      pst.Chars(pst#NL, 2)
   
      CASE AddrForm
          1  : bus.WriteChipA8(ChipAddr, RegAddr, Value)
          2  : bus.WriteChipA16(ChipAddr, RegAddr, Value)
          3  : bus.WriteChipA32(ChipAddr, RegAddr, Value)
          -1 : RETURN
        OTHER: pst.Str(String("Chose -1, 1, 2, or 3!"))
               pst.Chars(pst#NL, 2)

   pst.Str(String("Completed."))   
   pst.Chars(pst#NL, 2)         

DAT
           DaaaBbbbb    BYTE  "string_data",0        

{{

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and    
associated documentation files (the "Software"), to deal in the Software without restriction,        
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:                                                                 
                                                                                                     
The above copyright notice and this permission notice shall be included in all copies or substantial 
portions of the Software.                                                                            
                                                                                                     
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}                                    